/**
 * @file pboeservo.c
 * This is the main program start point.
 */
#include <stdio.h>
#include <stdlib.h>
#include <sys/thread.h>
#include <propeller.h>

#include "propboe.h"

/* These numbers define servo stop pulse width */
#define LEFT_CENTER 1500
#define RIGHT_CENTER 1500

/* stacks for COGs */
static int LeftServoStack  [STACK_SIZE];
static int RightServoStack [STACK_SIZE];

/* per-thread library variables */
static _thread_state_t LeftThreadData;
static _thread_state_t RightThreadData;

/* WheelServo COG variables */
int LeftServoSpeed;
int RightServoSpeed;
int LeftRotation;
int RightRotation;

static int MaxServoSpeed = 100;

/*
 * Servo COG function
 */
void LeftServoThread(void *arg __attribute__((unused)))
{
    int pinmask = (1<<LEFT_SERVO_PIN);
    int center = LEFT_CENTER;
    int ustime = CLKFREQ/1000000;

    DIRA |= pinmask;

    for(;;) {
        /* start with signal off */
        OUTA &= ~pinmask;

        if(LeftServoSpeed == 0) {
            waitcnt(ustime*center+CNT);
            LeftRotation = 0;
            continue;
        }

        if(LeftServoSpeed > MaxServoSpeed)
            LeftServoSpeed = MaxServoSpeed;
        if(LeftServoSpeed < -MaxServoSpeed)
            LeftServoSpeed = -MaxServoSpeed;

        /* move servo */
        LeftRotation = (-LeftServoSpeed/2*center)/MaxServoSpeed+center;
        waitcnt(ustime*center+CNT);
        OUTA |=  pinmask;
        waitcnt(ustime*LeftRotation+CNT);
        OUTA ^=  pinmask;
    }
}

/*
 * Servo COG function
 */
void RightServoThread(void *arg __attribute__((unused)))
{
    int pinmask = (1<<RIGHT_SERVO_PIN);
    int center = RIGHT_CENTER;
    int ustime = CLKFREQ/1000000;

    DIRA |= pinmask;

    for(;;) {
        /* start with signal off */
        OUTA &= ~pinmask;

        if(RightServoSpeed == 0) {
            waitcnt(ustime*center+CNT);
            RightRotation = 0;
            continue;
        }

        if(RightServoSpeed > MaxServoSpeed)
            RightServoSpeed = MaxServoSpeed;
        if(RightServoSpeed < -MaxServoSpeed)
            RightServoSpeed = -MaxServoSpeed;

        /* move servo */
        RightRotation = (RightServoSpeed/2*center)/MaxServoSpeed+center;
        waitcnt(ustime*center+CNT);
        OUTA |=  pinmask;
        waitcnt(ustime*RightRotation+CNT);
        OUTA ^=  pinmask;
    }
}


void startServoThreads(void)
{
    int n;
    n = _start_cog_thread(LeftServoStack + STACK_SIZE, LeftServoThread, NULL, &LeftThreadData);
    dprintf("Left Servo COG thread %d\n", n);
    n = _start_cog_thread(RightServoStack + STACK_SIZE, RightServoThread, NULL, &RightThreadData);
    dprintf("Right Servo COG thread %d\n", n);
}

void botStop(void)
{
    LeftServoSpeed = 0;
    RightServoSpeed = 0;
}

void botForward(int rate)
{
    LeftServoSpeed = rate;
    RightServoSpeed = rate;
}

void botReverse(int rate)
{
    LeftServoSpeed = -rate;
    RightServoSpeed = -rate;
}

void botLeftTurn(int rate)
{
    LeftServoSpeed = 0;
    RightServoSpeed = rate;
}

void botRightTurn(int rate)
{
    LeftServoSpeed = rate;
    RightServoSpeed = 0;
}

void botLeftSpin(int rate)
{
    LeftServoSpeed = rate;
    RightServoSpeed = -rate;
}

void botRightSpin(int rate)
{
    LeftServoSpeed = -rate;
    RightServoSpeed = rate;
}

/**
 * Main program function.
 */
int propBoeServoTest(void)
{
    int move = 's';
    int speed = 100;

    /* keep QuickStart TX quiet. */
    DIRA |= (1<<30);

    setbuf(stdout, 0);
    setbuf(stdin, 0);
    waitcnt(500*(CLKFREQ/1000)+CNT);

    startServoThreads();
    botStop();

    for(;;) {
        putchar(0);
        printf("PropBOE Servo Test\n");
        printf("Last Command '%c'. Speed %d\n", move, speed);
        printf("LeftRotation = %d ",LeftRotation);
        printf("RightRotation = %d\n",RightRotation);
        printf("LeftServoSpeed = %d ",LeftServoSpeed);
        printf("RightServoSpeed = %d\n",RightServoSpeed);
        printf("\n$ ");
        move = getchar();
        getchar();
        switch(move) {
            case 'f':
                botForward(speed);
                printf("forward %d\n", speed);
            break;
            case 'b':
                botReverse(speed);
                printf("backward %d\n", speed);
            break;
            case 'l':
                botLeftTurn(speed);
                printf("left  turn %d\n", speed);
            break;
            case 'r':
                botRightTurn(speed);
                printf("right turn %d\n", speed);
            break;
            case 'L':
                botLeftSpin(speed);
                printf("left  spin %d\n", speed);
            break;
            case 'R':
                botRightSpin(speed);
                printf("right spin %d\n", speed);
            break;
            case 's':
                botStop();
                printf("stop\n");
            break;
            case '+':
                speed += 100;
                printf("speed %d\n", speed);
            break;
            case '-':
                speed -= 100;
                printf("speed %d\n", speed);
            break;
        }

    }

    return 0;
}

